# -*- coding: utf-8 -*-


BOX_WIDTH = 80
BOX_HEIGHT = 3
PAD_WIDTH = 400
PAD_HEIGHT = 1000

ALI_BASE_URL = "http://47.93.19.8:3000"
LOCAL_BASE_URL = "http://127.0.0.1:3000"
API_URL = ALI_BASE_URL
# API_URL = LOCAL_BASE_URL


API_PLAY_INFO = API_URL + "/audio/{}"
API_RECENT = API_URL + "/audios/recent?page={}"
API_HOT_COMMENT = API_URL + "/audios/hot/comment?page={}"
API_HOT_LIKE = API_URL + "/audios/hot/like?page={}"
API_CATEGORY = API_URL + "/audios/category/{}?page={}"
API_TOPIC = API_URL + "/audios/topic/{}?page={}"
API_ALL_DJS = API_URL + "/audios/alldjs"
API_DJS = API_URL + "/audios/djs/{}?page={}"

CATE_DICT = {"Gadio News": 45,
             "Early Acess": 49,
             "Gadio Story": 53,
             "Gadio Life": 13,
             "Gadio Music": 14,
             "特别二次元": 38,
             "东京旮旯": 52,
             "葱丝推拿": 36,
             "圣地巡礼": 44,
             "广告节目": 62,
             }

TOPIC_DICT = {"辐射": "辐射",
              "战锤": "战锤",
              "魔兽": "魔兽",
              "通识": "通识"}

BASE_AUDIO_CATE_URL = "https://www.gcores.com/categories/9/originals"
BASE_AUDIO_LINK_URL = "https://www.gcores.com/radios/"
TEST_AUDIO_LINK = "https://www.gcores.com/radios/105962"



import curses
import datetime
import os
import sys
import threading
import time
import webbrowser

from gaycore.config import *
from gaycore.player import Player
from gaycore.utils import (API_CATEGORY_DICT, API_DJS_DICT, API_TOPIC_DICT,
                           BASE_AUDIO_CATE_URL, TOPIC_DICT,
                           async_make_name_mp3_dict, chunkstring,
                           hot_comment_func, hot_like_func, playinfo_func,
                           recent_func)

try:
    from urllib.request import urlretrieve
except:
    from urllib import urlretrieve


# Author: Yi Hong
# Date: 2019.01.29

MENU_DICT = {
    "最近电台": recent_func,
    "主播电台": API_DJS_DICT,
    "专题电台": API_TOPIC_DICT,
    "分类电台": API_CATEGORY_DICT,
    "排行电台": {"讨论热门": hot_comment_func, "喜欢热门": hot_like_func},
}


class GcoreBox:

    def __init__(self):
        self.BOX_WIDTH = BOX_WIDTH
        self.BOX_HEIGHT = BOX_HEIGHT

        self.PAD_WIDTH = PAD_WIDTH
        self.PAD_HEIGHT = PAD_HEIGHT
        self.windows = []
        self.flow_info = {}
        self.flow_img = ""
        self.quote = ""
        self.last_flag = False
        self.menu_dict = MENU_DICT
        self.dict_stack = []  # 定义一个stack 方便进入下个目录和返回下个目录
        self.page_num = -1  # 用于翻页
        self.player = Player()
        self._init_curses()

    def _init_curses(self):
        self.stdscr = curses.initscr()
        self.stdscr.keypad(1)
        curses.noecho()
        curses.cbreak()
        curses.curs_set(0)
        curses.start_color()
        try:
            curses.init_pair(1, curses.COLOR_BLACK, 197)  # 接近机核主题的颜色
        except:
            # 树莓派 windows无法使用机核like色
            curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_RED)
        curses.init_pair(2, curses.COLOR_WHITE, curses.COLOR_BLACK)
        self.stdscr.bkgd(curses.color_pair(2))
        self.stdscr.timeout(100)
        self.stdscr.refresh()

    def make_text_box(self, boxes):
        self.boxes = boxes
        self.pad = curses.newpad(self.PAD_WIDTH, self.PAD_HEIGHT)

        self.pad.box()
        height = 1
        for num, text in enumerate(boxes, 1):
            box = self.pad.derwin(
                self.BOX_HEIGHT, self.BOX_WIDTH, height, self.PAD_WIDTH//2 - self.BOX_WIDTH//2)
            box.box()
            if len(text) > 37:
                text = text[:36] + "..."
            box.addstr(1, 0, "{}. {}".format(num, text))
            yield box
            height += self.BOX_HEIGHT
        self.max_height = height

    def _center(self, window):
        cy, cx = window.getbegyx()
        maxy, maxx = self.stdscr.getmaxyx()
        self.pad.timeout(-1)
        self.pad.refresh(cy, cx, 1, maxx//2 -
                         self.BOX_WIDTH//2, maxy-1, maxx-1)
        return (cy, cx)

    def _end_curses(self):
        """ Terminates the curses application. """
        curses.nocbreak()
        self.stdscr.keypad(0)
        curses.echo()
        curses.endwin()

    def start_to_play(self, url):
        thread = threading.Thread(target=self.player.run_mpg123, args=(url, ))
        thread.start()
        time.sleep(0.01)
        return thread

    def start_to_download(self, mp3_url, audio_name):
        thread = threading.Thread(
            target=self._download_mp3, args=(mp3_url, audio_name))
        thread.start()

    def mainloop(self):

        topy, topx = self._center(self.windows[0])

        current_num = 0
        last = 1
        top = self.windows[0]
        while True:
            self.windows[current_num].bkgd(curses.color_pair(1))
            # 处理只有一个box的情况
            if len(self.windows) == 1:
                last = 0
            self.windows[last].bkgd(curses.color_pair(2))

            maxy, maxx = self.stdscr.getmaxyx()
            cy, cx = self.windows[current_num].getbegyx()

            if ((topy + maxy - self.BOX_HEIGHT) <= cy):
                top = self.windows[current_num]

            if topy >= cy + self.BOX_HEIGHT:
                top = self.windows[current_num]

            if last != current_num:
                last = current_num

            topy, topx = self._center(top)
            c = self.stdscr.getch()

            if c == curses.KEY_RESIZE:
                self.stdscr.clear()

            if c in [ord("j"), curses.KEY_DOWN]:
                current_num = current_num + \
                    1 if current_num < len(self.windows) - 1 else 0

            if c in [ord("k"), curses.KEY_UP]:
                current_num = current_num - \
                    1 if current_num > 0 else len(self.windows) - 1

            # enter the menu selected
            if c in [curses.KEY_RIGHT, ord("l")]:
                try:
                    if not callable(self.menu_dict):
                        self.page_num = -1  # 恢复翻页处理，翻页过多的问题
                        self.dict_stack.append(self.menu_dict)
                        self.menu_dict = self.menu_dict.get(
                            self.boxes[current_num])
                    if isinstance(self.menu_dict, dict):
                        self.boxes = list(self.menu_dict.keys())
                    else:
                        # 利用函数一等对象特性（可能需要重构）
                        if callable(self.menu_dict):
                            if self.last_flag:
                                break
                            self.dict_stack.append(self.menu_dict)
                            self.page_num += 1
                            self.info_dict = self.menu_dict(self.page_num)
                            self.boxes = list(self.info_dict.keys())
                            # 处理翻页到头情况
                            if len(self.boxes) < 10 and (not self.last_flag):
                                self.last_flag = True
                                self.last_flag_num = self.page_num
                        else:
                            self.boxes = self.menu_dict
                    break
                except:
                    self.dict_stack.pop()

            # back to last menu
            if c in [curses.KEY_LEFT, ord("h")]:
                try:
                    last_menu = self.dict_stack.pop()
                    if callable(last_menu):
                        # 处理翻页到头情况
                        if self.last_flag:
                            self.page_num = self.last_flag_num
                            self.last_flag = False
                        self.page_num = self.page_num - 1
                        if self.page_num >= 0:
                            self.info_dict = last_menu(self.page_num)
                            self.boxes = list(self.info_dict.keys())
                        else:
                            self.menu_dict = self.dict_stack.pop()
                            self.boxes = list(self.menu_dict.keys())
                    else:
                        self.menu_dict = last_menu
                        self.boxes = list(last_menu.keys())
                    break
                except:
                    pass
            # 退出
            if c == ord("q"):
                self._end_curses()
                self.player.q_mp3()
                return

            # 打开浏览器查看
            if c == ord("w"):
                try:
                    info_list = self.info_dict.get(self.boxes[current_num])
                    gcore_url = info_list[0]
                    webbrowser.open(gcore_url)
                except:
                    # 查看图片
                    webbrowser.open(self.flow_img)

            # 打开来源
            if c == ord("o"):
                try:
                    webbrowser.open(self.quote)
                except:
                    pass

            # 下载
            if c == ord("d"):
                if self.dict_stack and callable(self.dict_stack[-1]):
                    audio_name = self.boxes[current_num]
                    info_list = self.info_dict.get(audio_name)
                    # 下载当前MP3
                    self.start_to_download(info_list[1], audio_name + ".mp3")
                else:
                    continue

            if c == ord('\n'):
                try:
                    if self.player.popen_handler:

                        if current_num == len(self.windows) - 1:
                            webbrowser.open(self.flow_img)
                            continue
                        self.player.q_mp3()
                        self.windows.pop()
                    info_list = self.info_dict.get(self.boxes[current_num])
                    # 播放当前选择的音乐
                    self.start_to_play(info_list[1])
                    # 生成播放信息
                    audio_id = info_list[0].split("/")[-1]
                    playing_info = playinfo_func(audio_id)
                    time_flow_info = playing_info.get("audio_flow_info", "")
                    self.djs_info = playing_info.get("audio_djs", "")
                    self.flow_info = eval(time_flow_info)
                    # 添加播放时间轴
                    self._add_info_box()
                except:
                    pass

            # 暂停或继续播放
            if c == ord(" "):
                self.player.pause_or_resume_mp3()

            # 快进/倒退
            if c == ord("f"):
                self.player.forward_mp3(5)

            # 播放时间轴
            self._play_timeflow_info()

        self.pick(self.boxes)

    def _play_timeflow_info(self):

        # 解析主播格式
        def f_parser(x): return ', '.join([i[0] for i in eval(x)])

        # 播放时间轴
        if self.player.popen_handler:
            flow_info = dict(self.flow_info.get(
                self.player.process_length, ""))
            djs_info = f_parser(self.djs_info)[:50]
            self.windows[-1].box()
            self.windows[-1].addstr(1, 1, "时间轴")
            self.windows[-1].addstr(1, self.BOX_WIDTH//2-25, "主播:" + djs_info)
            # play_time = str(datetime.timedelta(
            #    seconds=self.player.process_length))
            play_time_hrs, play_time_min = divmod(
                self.player.process_location, 3600)
            play_time_min, play_time_sec = divmod(
                self.player.process_location, 60)

            self.windows[-1].addstr(3, 1, "time:  " + str(play_time_hrs).zfill(
                2) + ':' + str(play_time_min).zfill(2) + ':' + str(play_time_sec).zfill(2))
            if flow_info.get("content"):
                try:
                    self.flow_img = flow_info.get("img", "")
                    self.quote = flow_info.get("quote_href", "")
                    self.windows[-1].clear()
                    self.windows[-1].bkgd(curses.color_pair(2))
                    self.windows[-1].addstr(4, 1, "title: " +
                                            flow_info.get("title", ""))
                    self.windows[-1].addstr(5, 1, "content:")
                    content = flow_info.get("content", "")
                    # 去掉换行符
                    content = content.replace("\n", "")
                    content_length = len(content)
                    if content_length > 230:
                        content = content[:230] + "..."
                    content_list = chunkstring(content, 37)
                    for i, j in enumerate(content_list):
                        self.windows[-1].addstr(6 + i, 1, "  " + j)
                    if self.quote:
                        self.windows[-1].addstr(13, 1, "打开来源请按o键")
                except:
                    pass

    def _add_info_box(self):
        box = self.pad.derwin(self.BOX_HEIGHT*5, self.BOX_WIDTH,
                              self.max_height, self.PAD_WIDTH//2 - self.BOX_WIDTH//2)
        box.box()
        self.windows.append(box)

    def _download_mp3(self, mp3_url, audio_name):

        # 下载进度
        def report(count, blockSize, totalSize):
            percent = int(count*blockSize*100/totalSize)
            self.stdscr.addstr(0, 0, "\r%d%%" % percent + ' complete')
        try:
            urlretrieve(mp3_url, audio_name, reporthook=report)
            self.stdscr.addstr(0, 0, audio_name + ': 下载完成')
            time.sleep(1)
            self.stdscr.clear()
        except:
            self.stdscr.addstr(0, 0, "download error")

    def pick(self, boxes):
        self.windows = list(self.make_text_box(boxes))
        if self.player.popen_handler:
            self._add_info_box()
        self.mainloop()



# -*- coding: utf-8 -*-

import time
import traceback
from sqlalchemy import create_engine
from sqlalchemy import Column, Integer, String, TEXT, DATE, SmallInteger
from sqlalchemy.dialects.mysql import LONGTEXT
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, mapper
from gaycore.spider import *
from gaycore.utils import *

Base = declarative_base()
engine = create_engine("mysql+pymysql://root@127.0.0.1:3306/gcore?charset=utf8mb4")
Session = sessionmaker(engine)
session = Session()


class Audio(Base):

    __tablename__ = "gcore_audio"

    id = Column(Integer, primary_key=True)
    audio_id = Column("audio_id", Integer, unique=True, nullable=False)
    audio_name = Column("audio_name", String(1000), nullable=False)
    audio_date = Column("audio_date", DATE, nullable=False)
    audio_url = Column("audio_url", String(200), nullable=False)
    audio_mp3_url = Column("audio_mp3_url", String(1000), nullable=False)
    audio_flow_info = Column("audio_flow_info", LONGTEXT, nullable=False, default="[]")
    audio_djs = Column("audio_djs", String(200), nullable=False, default="[]")
    audio_like = Column("audio_like", SmallInteger, nullable=False, default=0)
    audio_comment = Column("audio_comment", SmallInteger, nullable=False, default=0)
    audio_cate = Column("audio_cate", String(200), nullable=False)
    audio_cate_name = Column("audio_cate_name", String(200), nullable=False)


Base.metadata.create_all(engine)


name_list = ["audio_id",
             "audio_name",
             "audio_date",
             "audio_url",
             "audio_mp3_url",
             "audio_flow_info",
             "audio_djs",
             "audio_like",
             "audio_comment",
             "audio_cate",
             "audio_cate_name",
             ]


def paser_value(value):
    audio_id = value[3].split("/")[-1]
    audio_name = value[0]
    audio_date = value[1][0]
    audio_url = value[3]
    audio_mp3_url = value[4]
    audio_flow_info = value[6]
    audio_djs = value[5]
    audio_like = value[2][0]
    audio_comment = value[2][1]
    audio_cate = value[1][1]
    audio_cate_name = value[1][2]
    return [audio_id, audio_name, audio_date, audio_url, audio_mp3_url, audio_flow_info,
            audio_djs, audio_like, audio_comment, audio_cate, audio_cate_name]


def get_page_data(page):
    url = BASE_AUDIO_CATE_URL + "?page={}".format(page)
    cate_result = get_one_cate_info(url)
    insert_data = [dict(zip(name_list, paser_value(i))) for i in cate_result]
    data_list = [Audio(**i) for i in insert_data]
    return data_list


def insert_one_page(page):
    data_list = get_page_data(page)
    session.add_all(data_list)
    try:
        session.commit()
        # session.merge()
    except:
        traceback.print_exc()
        session.rollback()
        time.sleep(1)


def insert_all_page(start, end):
    for i in range(start, end):
        try:
            insert_one_page(i)
            print(i)
        except Exception as e:
            time.sleep(2)
            try:
                insert_one_page(i)
            except:
                traceback.print_exc()
                print("{} is wrong".format(i))


def daily_spider(page=0):
    old_id_list = session.query(Audio.audio_id).order_by(Audio.audio_id.desc()).limit(30)
    old_id_list = [str(i.audio_id) for i in old_id_list]
    data_list = get_page_data(page)
    spider_id_list = [(i.audio_id, i.audio_like, i.audio_comment) for i in data_list]
    spider_old_id_list = [i for i in spider_id_list if i[0] in old_id_list]
    spider_new_id_list = list(set([i[0] for i in spider_id_list]) - set([i[0] for i in spider_old_id_list]))

    # new audios to insert
    insert_values = [i for i in data_list if i.audio_id in spider_new_id_list]
    session.add_all(insert_values)
    session.commit()

    # old audios to update
    for i in spider_old_id_list:
        session.query(Audio).filter(Audio.audio_id==i[0]).update({"audio_like": i[1], "audio_comment": i[2]})
        session.commit()



import curses
import time
import subprocess


class Player:

    def __init__(self):
        self.popen_handler = None
        self.process_location = 0
        self.process_length = 0
        self.fps = 0

    # 参考musicibox代码
    def run_mpg123(self, url, expires=-1, get_time=-1):
        para = ["mpg123", "-R"] + []
        self.popen_handler = subprocess.Popen(
            para, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

        self.popen_handler.stdin.write(b"L " + url.encode("utf-8") + b"\n")
        self.popen_handler.stdin.flush()

        endless_loop_cnt = 0
        # some code from musicbox
        while True:
            if not self.popen_handler:
                break

            strout = self.popen_handler.stdout.readline().decode("utf-8").strip()
            if strout[:2] == "@F":
                out = strout.split(" ")
                self.process_location = int(float(out[3]))
                self.process_length = int(float(out[3]) + float(out[4]))
                if(float(out[3]) != 0):
                    self.fps = float(out[1]) / float(out[3])

            elif strout == "@P 0":
                # end, moving to next
                playing_flag = True
                break
            elif strout == "":
                endless_loop_cnt += 1
                if endless_loop_cnt > 100:
                    break

    # 停止播放
    def pause_or_resume_mp3(self):
        if not self.popen_handler:
            return
        self.popen_handler.stdin.write(b"P\n")
        self.popen_handler.stdin.flush()

    # 快进/倒退10s
    def forward_mp3(self, seconds):
        if not self.popen_handler:
            return
        self.popen_handler.stdin.write(
            b"J +" + str(seconds*int(self.fps)).encode("utf-8") + b"\n")
        self.popen_handler.stdin.flush()

    # 退出播放
    def q_mp3(self):
        if self.popen_handler:
            self.popen_handler.stdin.write(b"Q\n")
            self.popen_handler.stdin.flush()
            self.popen_handler.kill()
            self.popen_handler = None



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

VERSION = '0.1.6'

setup(name='gaycore',
      version=VERSION,
      description="a tiny and smart cli player of gcore audio, based on Python package curses and mpg123 for Linux or Mac",
      long_description='gaycore listen player using cli enjoy it',
      keywords='python gcore gaycore cli terminal',
      author='yihong0618',
      author_email='zouzou0208@gmail.com',
      url='https://github.com/yihong0618/gaycore',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=True,
      install_requires=[
          'requests', 'aiohttp', 'beautifulsoup4', 'PyMySQL', 'SQLAlchemy'
      ],
      classifiers=[
          "Development Status :: 4 - Beta",
          "Environment :: Console",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Topic :: Software Development :: Libraries",
      ],
      entry_points={
          'console_scripts': [
              'gaycore = gaycore.main:run'
          ], }
      )



#!/usr/bin/env python # encoding: utf-8

import time
import json
import re
from collections import Counter
import asyncio
from functools import partial
import aiohttp
import requests
from bs4 import BeautifulSoup as Soup
from gaycore.spider import *
from gaycore.config import *


FLOW_PATTERN = re.compile(r"timelines:(.*)jplayerSwf", re.DOTALL)


# func to format the content
def chunkstring(string, length):
    return [string[0+i:length+i] for i in range(0, len(string), length)]


def make_soup(url):
    try:
        response = requests.get(url).text
        soup = Soup(response, "html.parser")
    except:
        soup = None
    return soup


def get_mp3_url(soup):
    https_url = soup.find("p", class_="story_actions").find("a").attrs["href"]
    # replace https to http for mpg123
    return https_url.replace("https", "http")


def make_name_mp3_dict(url):
    soup = make_soup(url)
    if not soup:
        return dict()

    audio_list = soup.find_all("div", class_="showcase_img")
    audio_name_list = [i.img.attrs['alt'] for i in audio_list]
    audio_link_list = [i.find("a").attrs["href"] for i in audio_list]
    mp3_link_list = [get_mp3_url(make_soup(i)) for i in audio_link_list]
    return dict(zip(audio_name_list, zip(audio_link_list, mp3_link_list)))


def get_djs_info(soup):
    try:
        djs = soup.find("div", class_="story_djs_items")
        if not djs:
            djs_list = []
        djs_text = djs.get_text().replace("\n", "")
        djs_text = re.sub(" +", " ", djs_text)
        djs_list = djs_text.split()
        return djs_list
    except:
        return []


def get_timeflow_info(soup):
    div = soup.find_all("div")[-1]
    src = div.find("script", type="text/javascript").text
    try:
        flow_info = re.search(FLOW_PATTERN, src).group(1)
    except:
        flow_info = ''  # todo
    # to make a info list an fmt it
    if not flow_info:
        return []
    flow_info = flow_info.lstrip()
    flow_info = flow_info.rstrip()
    flow_info = flow_info[:-1]
    # 此处处理json中的null null无法eval获取
    try:
        flow_info_list = eval(flow_info, {"null": "", "false": "", "true": ""})
    except:
        flow_info_list = []
    return flow_info_list


def paser_showcase(soup):
    date = soup.get_text()
    audio_category = soup.find('a')['href']
    audio_category_name = soup.find('a').get_text()
    return date, audio_category, audio_category_name


def get_showcase_info(soup):
    name = soup.find('h1', class_="story_title").get_text()
    name = info_paser(name)
    mp3_url = get_mp3_url(soup)
    djs_info = str(get_djs_info(soup))
    time_flow_info = get_timeflow_info(soup)
    time_flow_info = parsr_timeflow_info(time_flow_info)
    return name, mp3_url, time_flow_info, djs_info


async def fetch_data(url):
    async with aiohttp.ClientSession() as session:
        async with session.get(url, timeout=5) as resp:
            r = await resp.text()
            name, mp3_url, time_flow_info, djs_info = get_showcase_info(
                Soup(r, "html.parser"))
            return name, (url, mp3_url, time_flow_info, djs_info)


def async_make_name_mp3_dict(url):
    url_list = get_all_page_link(make_soup(url))
    event_loop = asyncio.get_event_loop()
    tasks = [fetch_data(i) for i in url_list]
    results = event_loop.run_until_complete(asyncio.gather(*tasks))
    return dict(results)


def api_make_name_mp3_dict(api, name='', page=0):

    results_dict = {}
    info_json = requests.get(api.format(name, page)).text
    info_list = json.loads(info_json).get("result", [])
    for info in info_list:
        results_dict[info.get("audio_name")] = (
            info.get("audio_url", ""), info.get("audio_mp3_url", ""))
    return results_dict


def api_get_play_info(api, audio_id):
    info_json = requests.get(api.format(audio_id)).text
    info_list = json.loads(info_json).get("result", [])
    return info_list


def make_base_dict_func(BASE_DICT, API):
    category_dict = {}
    for cate_name, cate_name_num in BASE_DICT.items():
        category_dict[cate_name] = partial(
            api_make_name_mp3_dict, API, cate_name_num)
    return category_dict


def get_top_djs(limit=50):
    info_json = requests.get(API_ALL_DJS).text
    result = json.loads(info_json).get("result", [])
    result = [eval(i) for i in result]

    def make_result():
        for i in result:
            for j in i:
                yield j

    top_djs = Counter(list(make_result())).most_common(limit)
    return dict(top_djs)


def make_djs_dict_func(top_djs):
    djs_dict = {}
    for i, j in top_djs.items():
        djs_dict[i[0]+"    电台数量" +
                 str(j)] = partial(api_make_name_mp3_dict, API_DJS, i[0])
    return djs_dict


# 主播字典
API_DJS_DICT = make_djs_dict_func(get_top_djs())

# 分类字典
API_CATEGORY_DICT = make_base_dict_func(CATE_DICT, API_CATEGORY)
API_TOPIC_DICT = make_base_dict_func(TOPIC_DICT, API_TOPIC)

recent_func = partial(api_make_name_mp3_dict, API_RECENT)
hot_comment_func = partial(api_make_name_mp3_dict, API_HOT_COMMENT)
hot_like_func = partial(api_make_name_mp3_dict, API_HOT_LIKE)
playinfo_func = partial(api_get_play_info, API_PLAY_INFO)



# -*- coding: utf-8 -*-
#!/usr/bin/env python

import json
import re
import time
from collections import defaultdict
import asyncio
import requests
from bs4 import BeautifulSoup as Soup
import aiohttp

BASE_AUDIO_CATE_URL = "https://www.gcores.com/categories/9/originals"
BASE_AUDIO_LINK_URL = "https://www.gcores.com/radios/"
TEST_AUDIO_LINK = "https://www.gcores.com/radios/105962"
UPLOAD_BASE_URL = "https://alioss.gcores.com"
headers = {
    'User-agent': "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36",
}

flow_pattern = re.compile(r"timelines:(.*)jplayerSwf", re.DOTALL)


def make_soup(url):
    try:
        response = requests.get(url).text
        soup = Soup(response, "html.parser")
    except:
        soup = None
    return soup


def get_mp3_url(soup):
    https_url = soup.find("p", class_="story_actions").find("a").attrs["href"]
    return https_url.replace("https", "http")  # replace https to http for mpg123


def get_all_page_link(soup):
    if not soup:
        return   # to do

    audio_list = soup.find_all("div", class_="showcase_img")
    audio_link_list = [i.find("a").attrs["href"] for i in audio_list]
    return audio_link_list


def make_name_mp3_dict(soup):
    if not soup:
        return dict()

    audio_list = soup.find_all("div", class_="showcase_img")
    audio_name_list = [i.img.attrs['alt'] for i in audio_list]
    audio_link_list = [i.find("a").attrs["href"] for i in audio_list]
    mp3_link_list = [get_mp3_url(i) for i in audio_link_list]
    time_flow_list = [get_timeflow_info(i) for i in audio_link_list]
    return dict(zip(audio_name_list, zip(audio_link_list, mp3_link_list)))


def info_paser(string):
    text = string.replace("\n", "")
    text = re.sub(" +", " ", text)
    text = text.lstrip().rstrip()
    return text


def get_djs_id(soup):
    try:
        djs_info = soup.find("div", class_="story_djs_items")
        djs_info = djs_info.find_all("a")
        djs_id_link = [i["href"] for i in djs_info]
        djs_id = [i.split("/")[-1] for i in djs_id_link]
    except:
        djs_id = []
    return djs_id


def get_djs_info(soup):
    try:
        djs = soup.find("div", class_="story_djs_items")
        if not djs:
            djs_list = []
        djs_text = djs.get_text()
        djs_text = info_paser(djs_text)
        djs_name = djs_text.split()
        djs_id = get_djs_id(soup)
        djs_info = list(zip(djs_name, djs_id))
        return djs_info
    except:
        return []


def get_timeflow_info(soup):
    div = soup.find_all("div")[-1]
    src = div.find("script", type="text/javascript").text
    try:
        flow_info = re.search(flow_pattern, src).group(1)
    except:
        flow_info = ''  # todo
    # to make a info list an fmt it
    if not flow_info:
        return []
    flow_info = flow_info.lstrip()
    flow_info = flow_info.rstrip()
    flow_info = flow_info[:-1]
    # 此处处理json中的null null无法eval获取
    try:
        flow_info_list = eval(flow_info, {"null": "", "false": "", "true": ""})
    except:
        flow_info_list = []
    return flow_info_list


def showcase_paser(soup):
    date = soup.get_text()
    date = info_paser(date).split()[-1]
    audio_category = soup.find('a')['href']
    audio_category = info_paser(audio_category)
    audio_category_name = soup.find('a').get_text()
    audio_category_name = info_paser(audio_category_name)
    return date, audio_category, audio_category_name


def meta_num_paser(soup):
    audio_meta_num = soup.get_text()
    audio_meta_num = info_paser(audio_meta_num)
    like, comment = audio_meta_num.split()[:-1]
    return like, comment


def get_cate_audio_info(soup):
    if not soup:
        return dict()

    audio_list = soup.find_all("div", class_="showcase_img")
    audio_name_list = [i.img.attrs['alt'] for i in audio_list]
    audio_link_list = [i.find("a").attrs["href"] for i in  audio_list]
    audio_showcase = soup.find_all("div", class_="showcase_time")
    audio_showcase_list = [showcase_paser(i) for i in audio_showcase]
    audio_meta_num = soup.find_all("div", class_="showcase_meta")
    audio_meta_num_list = [meta_num_paser(i) for i in audio_meta_num]
    return dict(zip(audio_link_list, list(zip(audio_name_list, audio_showcase_list, audio_meta_num_list))))


# 获取一个url的信息
def get_info_audio_url(soup):
    mp3_url = get_mp3_url(soup)
    flow_info = get_timeflow_info(soup)
    djs_info = get_djs_info(soup)
    return mp3_url, djs_info, flow_info


def parsr_timeflow_info(flow_info):
    # parser the gcores timeflow info to {time: [title, content, quate_href]} dict-list dict for curses to display
    time_flow_dict = defaultdict(dict)
    for i in flow_info:
        flow_time_key = i.get("at")
        time_flow_dict[flow_time_key]["title"] = i.get("title", "")
        time_flow_dict[flow_time_key]["content"] = i.get("content", "")
        time_flow_dict[flow_time_key]["quote_href"] = i.get("quote_href", "")
        time_flow_dict[flow_time_key]["img"] = UPLOAD_BASE_URL + i.get("asset", {}).get("limit", {}).get("url", "")
    return dict(time_flow_dict)


async def fetch_gcore_data(url):
    async with aiohttp.ClientSession() as session:
        async with session.get(url, headers=headers) as resp:
            r = await resp.text()
            soup = Soup(r, "html.parser")
            djs_info = str(get_djs_info(soup))
            mp3_url = get_mp3_url(soup)
            time_flow_info = get_timeflow_info(soup)
            time_flow_info = str(parsr_timeflow_info(time_flow_info))
            return url, mp3_url, djs_info, time_flow_info


# 获取一个cate的信息
def get_one_cate_info(url):
    event_loop = asyncio.get_event_loop()
    soup = make_soup(url)
    sub_url_list = get_all_page_link(soup)
    tasks = [fetch_gcore_data(u) for u in sub_url_list]
    results = event_loop.run_until_complete(asyncio.gather(*tasks))
    # 获取该cate_url下的所有info
    cate_info_dict = get_cate_audio_info(soup)
    results = [cate_info_dict.get(i[0]) + i for i in results]
    return results

# print(get_one_cate_info(BASE_AUDIO_CATE_URL)[0])
